@@ -40,10 +40,8 @@ def register():
         Configuration,
         Period,
         JournalType,
-        JournalView,
-        JournalViewColumn,
         Journal,
-        OpenJournalCashStart,
+        JournalCashContext,
         JournalPeriod,
         Move,
         Reconciliation,
@@ -71,6 +69,8 @@ def register():
         AccountTemplate2,
         AccountTax,
         Account2,
+        TestTaxView,
+        TestTaxViewResult,
         MoveTemplate,
         MoveTemplateKeyword,
         MoveLineTemplate,
@@ -88,7 +88,6 @@ def register():
         UpdateChart,
         ClosePeriod,
         ReOpenPeriod,
-        OpenJournalCash,
         CloseJournalPeriod,
         ReOpenJournalPeriod,
         OpenJournal,
@@ -101,6 +100,8 @@ def register():
         CreateMove,
         OpenChartTaxCode,
         OpenTaxCode,
+        TestTax,
+        PartyReplace,
         module='account', type_='wizard')
     Pool.register(
         GeneralLedger,
